@@ -2,7 +2,7 @@
 Module for keeping exhibit-wide constants
 """
 
-ORIGINAL_VALUES_DB = "Number of unique values is above category threshold"
+ORIGINAL_VALUES_DB = "Number of unique values is above inline limit"
 ORIGINAL_VALUES_PAIRED = "See paired column"
 ORIGINAL_VALUES_REGEX = "regex"
 MISSING_DATA_STR = "Missing data"
\ No newline at end of file
