@@ -2,7 +2,7 @@
 Module for various derived and user-set constraints
 '''
 # Standard library imports
-from collections import namedtuple, Counter
+from collections import namedtuple
 from datetime import datetime
 from functools import partial
 import itertools as it
@@ -17,6 +17,7 @@ from .sql import query_anon_database
 from .generate.continuous import scale_continuous_column
 from .constants import ORIGINAL_VALUES_DB, ORIGINAL_VALUES_PAIRED, MISSING_DATA_STR
 from .generate.geo import geo_make_regions
+from .linkage.matrix import generate_user_linked_anon_df
 
 class ConstraintHandler:
     '''
@@ -132,6 +133,8 @@ class ConstraintHandler:
 
         source = self.input if self.output is None else self.output
         output_df = source.copy()
+        # preserve category dtypes at the end of each custom action
+        output_dtypes = output_df.dtypes[output_df.dtypes == "category"]
 
         dispatch_dict = {
             "make_outlier"         : self.make_outlier,
@@ -139,6 +142,7 @@ class ConstraintHandler:
             "sort_descending"      : partial(self.sort_values, asc=False),
             "make_distinct"        : self.make_distinct,
             "make_same"            : self.make_same,
+            "make_almost_same"     : partial(self.make_same, almost=True),
             "generate_as_sequence" : self.generate_as_sequence,
             "generate_as_repeating_sequence" :
                         partial(self.generate_as_sequence, repeating=True),
@@ -188,8 +192,7 @@ class ConstraintHandler:
                         # overwrite the original DF row IDs with the adjusted ones
                         output_df.loc[cc_filter_idx] = action_func(
                             output_df, cc_filter_idx, target_str,
-                            cc_partitions, **_kwargs)
-
+                            cc_partitions, **_kwargs).astype(output_dtypes)
         return output_df
 
     def adjust_dataframe_to_fit_constraint(self, anon_df, bool_constraint):
@@ -478,7 +481,10 @@ class ConstraintHandler:
 
             if partition_cols is not None:
 
-                grouped_series = df.dropna(subset=[target_col]).groupby(partition_cols)[target_col]
+                grouped_series = (
+                    df
+                    .dropna(subset=[target_col])
+                    .groupby(partition_cols, observed=True)[target_col])
                 outlier_series = grouped_series.transform(_within_group_outliers)
                 result = series.copy()
                 result.loc[filter_idx] = outlier_series.loc[filter_idx]
@@ -520,7 +526,7 @@ class ConstraintHandler:
             final_result + 
             [df.loc[filter_idx, [x for x in df.columns if x not in target_cols]]],
             axis=1
-        )
+        ).reindex(columns=df.columns)
 
         return new_df
 
@@ -567,7 +573,7 @@ class ConstraintHandler:
             # sorted(False) = ascending; df.sort_values(False) = descending
             # meaning for sorted() we have to reverse the boolean parameter
             result = (df
-                .groupby(partition_cols)[target_col]
+                .groupby(partition_cols, observed=True)[target_col]
                 .transform(sorted, reverse=not asc)
                 .loc[filter_idx]
             )
@@ -579,7 +585,7 @@ class ConstraintHandler:
             final_result + 
             [df.loc[filter_idx, [x for x in df.columns if x not in target_cols]]],
             axis=1
-        )
+        ).reindex(columns=df.columns)
 
         return new_df
 
@@ -661,7 +667,7 @@ class ConstraintHandler:
 
             result = (df
                 .loc[filter_idx]
-                .groupby(partition_cols)[target_col]
+                .groupby(partition_cols, observed=True)[target_col]
                 .transform(_make_distinct_within_group)
             )
 
@@ -672,12 +678,13 @@ class ConstraintHandler:
             final_result + 
             [df.loc[filter_idx, [x for x in df.columns if x not in target_cols]]],
             axis=1
-        )
+        ).reindex(columns=df.columns)
 
         return new_df
 
     def make_same(
-        self, df, filter_idx, target_str, partition_cols=None):
+        self, df, filter_idx, target_str,
+        partition_cols=None, almost=False, cascade=True):
         '''
         Force all values in the partition to be the same as the first.
         Remember that groubpy doesn't sort the observations within groups
@@ -694,13 +701,29 @@ class ConstraintHandler:
             Column(s) where user wants to make same
         partition_cols : list
             Columns to group by to achieve nested sort
+        almost         : Boolean
+            Generate values that are the same in 95% of rows
+        cascade        : Boolean
+            Re-generate linked columns if target column is part of a linked group
 
         Returns
         -------
         pd.DataFrame
             Only the filtered data slice with identical values is returned
         '''
-       
+
+        def _make_almost_same(group):
+
+            rng = self.rng.random(size=len(group))
+            uniques = sorted(group.unique())
+            same_val = uniques[0]
+            rng_val = same_val if len(uniques) == 1 else self.rng.choice(uniques[1:])
+
+            result_arr = np.where(rng < 0.05, rng_val, same_val)
+            result_series = pd.Series(
+                data=result_arr, index=group.index, dtype=group.dtype, name=group.name)
+            return result_series
+
         final_result = []
         target_cols = [x.strip() for x in target_str.split(",")]
         if partition_cols is not None:
@@ -710,14 +733,19 @@ class ConstraintHandler:
 
             if partition_cols is None:
                 
+                # without groupby, transform / apply will pass a single cell string
+                # instead of the series to the function.
                 repl = df.loc[filter_idx[0], target_col]
-                new_series = df.loc[filter_idx, target_col].transform(lambda x: repl)
+                s = df.loc[filter_idx, target_col]
+                new_series = (
+                    _make_almost_same(s) if almost else s.transform(lambda x: repl))
                 final_result.append(new_series)
                 continue
 
+            transform_func = _make_almost_same if almost else lambda x: x.iloc[0]
             temp_result = (df
-                .groupby(partition_cols)[target_col]
-                .transform(lambda x: x.iloc[0])
+                .groupby(partition_cols, observed=True)[target_col]
+                .transform(transform_func)
                 .loc[filter_idx]
             )
 
@@ -728,9 +756,42 @@ class ConstraintHandler:
             final_result + 
             [df.loc[filter_idx, [x for x in df.columns if x not in target_cols]]],
             axis=1
-        )
+        ).reindex(columns=df.columns)
 
-        return new_df
+        # should really look at changing the linked groups spec to being a dict
+        user_linked_cols = []
+        try:
+            if self.spec_dict["linked_columns"][0][0] == 0:
+                user_linked_cols = self.spec_dict["linked_columns"][0][1]
+        except (IndexError, KeyError):
+            return new_df
+
+        # at the end, if any of the target cols are in the user linked cols,
+        # re-create the ulinked df
+        if cascade and (set(target_cols) & set(user_linked_cols)):
+            
+            # make sure the starting matrix has object dtype because it can have a mix
+            # of values inside: dependening on what's been pre-generated
+            starting_col_matrix = np.full(
+                shape=(new_df.shape[0], len(user_linked_cols)),
+                fill_value=-1).astype("O")
+
+            idx = [user_linked_cols.index(target_col) for target_col in target_cols]
+
+            starting_col_matrix[:, idx] = new_df[target_cols].values
+
+            ulinked_df = generate_user_linked_anon_df(
+                self.spec_dict, user_linked_cols, new_df.shape[0], starting_col_matrix)
+
+            new_df = pd.concat(
+                [ulinked_df.set_index(new_df.index)] + 
+                [df.loc[filter_idx, [x for x in df.columns if x not in user_linked_cols]]],
+                axis=1
+            ).reindex(columns=df.columns)
+
+            return new_df
+
+        return new_df #pragma: no cover
 
     def generate_as_sequence(
         self, df, filter_idx, target_str, partition_cols=None, repeating=False):
@@ -851,17 +912,17 @@ class ConstraintHandler:
                 
                 new_vals = _generate_ordered_values(
                     filter_idx, ordered_list, ordered_probs)
-                final_result.append(pd.Series(new_vals, index=filter_idx))
+                final_result.append(
+                    pd.Series(new_vals, index=filter_idx, name=target_col))
                 continue
 
             result = (df
-                .groupby(partition_cols)[target_col]
+                .groupby(partition_cols, observed=True)[target_col]
                 .transform(
                     _generate_ordered_values,
                     ordered_list=ordered_list,
                     ordered_probs=ordered_probs,
                 )
-                .loc[filter_idx]
             )
 
             final_result.append(result)
@@ -871,7 +932,7 @@ class ConstraintHandler:
             final_result + 
             [df.loc[filter_idx, [x for x in df.columns if x not in target_cols]]],
             axis=1
-        )
+        ).reindex(columns=df.columns)
 
         return new_df
 
@@ -986,10 +1047,10 @@ class ConstraintHandler:
 
             # add nulls based on the miss_probability of the skew column
             miss_pct = self.spec_dict["columns"][skew_col]["miss_probability"]
-
+            miss_val = pd.NA if group.dtype =='Int64' else np.nan
             skewed_result = np.where(
                 rng.random(size=nrows) < miss_pct,
-                pd.NA, result.values)
+                miss_val, result.values)
 
             skewed_series = pd.Series(
                 data=skewed_result, index=group.index, name=skew_col)
@@ -1022,19 +1083,19 @@ class ConstraintHandler:
                 [skewed_series, sorted_series] + 
                 [df.loc[filter_idx, [x for x in df.columns if x not in target_cols]]],
                 axis=1
-            )
+            ).reindex(columns=df.columns)
 
             return new_df
 
         skewed_series = (df
             .loc[filter_idx]
-            .groupby(partition_cols)[skew_col]
+            .groupby(partition_cols, observed=True)[skew_col]
             .transform(_make_skewed_series)
         )
 
         sorted_series = (df
                 .loc[filter_idx]
-                .groupby(partition_cols)[sort_col]
+                .groupby(partition_cols, observed=True)[sort_col]
                 .transform(sorted)  
             )
 
@@ -1042,7 +1103,7 @@ class ConstraintHandler:
             [skewed_series, sorted_series] + 
             [df.loc[filter_idx, [x for x in df.columns if x not in target_cols]]],
             axis=1
-        )
+        ).reindex(columns=df.columns)
 
         return new_df        
 
