@@ -10,10 +10,13 @@ from collections import UserDict
 from pathlib import Path
 
 # External library imports
+import dill
 import yaml
 import numpy as np
 import pandas as pd
 
+from exhibit.core.constants import MISSING_DATA_STR
+
 # Exhibit imports
 from .specs import newSpec
 from .formatters import parse_original_values
@@ -21,7 +24,7 @@ from .validator import newValidator
 from .constraints import ConstraintHandler
 from .utils import (
     path_checker, read_with_date_parser, count_core_rows,
-    get_attr_values)
+    get_attr_values, package_dir)
 
 from .generate.missing import MissingDataGenerator
 from .generate.categorical import CategoricalDataGenerator
@@ -91,8 +94,8 @@ class newExhibit:
         self, command, source, output=None,
         inline_limit=30, equal_weights=False,
         skip_columns=None, linked_columns=None, 
-        uuid_columns=None, discrete_columns=None, 
-        verbose=False):
+        uuid_columns=None, discrete_columns=None,
+        save_probabilities=None, verbose=False, **kwargs):
         '''
         Initialise either from the CLI or by instantiating directly
         '''
@@ -110,6 +113,7 @@ class newExhibit:
         self.linked_columns= linked_columns or set()
         self.uuid_columns= uuid_columns or set()
         self.discrete_columns = discrete_columns or set()
+        self.save_probabilities = save_probabilities or set()
         self.verbose = verbose
 
         self.spec_dict = None
@@ -152,6 +156,7 @@ class newExhibit:
                 ew=self.equal_weights,
                 user_linked_cols=self.linked_columns,
                 uuid_cols=self.uuid_columns,
+                save_probabilities=self.save_probabilities
                 )
 
             self.spec_dict = new_spec.output_spec_dict()
@@ -192,14 +197,19 @@ class newExhibit:
         anonymised values from anon db in the generation process.
         '''
 
-        if not isinstance(self.source, Path): #pragma: no cover
-            self.source = path_checker(self.source)
+        # for internal testing, you can pass a spec_dict directly as spec rather than
+        # having to create a .yml first in a temp directory and read it.
+        if isinstance(self.source, dict): #pragma: no cover
+            self.spec_dict = self.source
+        else:
+            if not isinstance(self.source, Path): #pragma: no cover
+                self.source = path_checker(self.source)
 
-        if self.source.suffix == ".yml":
-            with open(self.source) as f:
-                self.spec_dict = yaml.safe_load(f)
-        else: #pragma: no cover
-            raise TypeError("Specification is not in .yml format")
+            if self.source.suffix == ".yml":
+                with open(self.source) as f:
+                    self.spec_dict = yaml.safe_load(f)
+            else: #pragma: no cover
+                raise TypeError("Specification is not in .yml format")
 
         # for convenience, some YAML heading, like numerical_columns or linked_columns
         # can be left blank by the user; this is read as NONE by YAML parser, but the
@@ -212,6 +222,9 @@ class newExhibit:
         if self.spec_dict.get("linked_columns", None) is None:
                 self.spec_dict["linked_columns"] = list()
 
+        if self.spec_dict.get("derived_columns", None) is None:
+                self.spec_dict["derived_columns"] = dict()
+
         for col in self.spec_dict["metadata"]["categorical_columns"]:
 
             original_values = self.spec_dict["columns"][col]["original_values"]
@@ -250,6 +263,10 @@ class newExhibit:
         #2) GENERATE CATEGORICAL PART OF THE DATASET (INC. TIMESERIES)
         cat_gen = CategoricalDataGenerator(self.spec_dict, core_rows)
         anon_df = cat_gen.generate()
+        for cat_col in self.spec_dict["metadata"]["categorical_columns"]:
+            anon_df[cat_col] = anon_df[cat_col].astype("category")
+            if MISSING_DATA_STR not in anon_df[cat_col].cat.categories:
+                anon_df[cat_col] = anon_df[cat_col].cat.add_categories(MISSING_DATA_STR)
 
         #3) ADD CONTINUOUS VARIABLES TO ANON DF
         # at this point, we don't have any Missing data placeholders (or actual nans)
@@ -265,7 +282,7 @@ class newExhibit:
         for num_col in self.spec_dict["metadata"]["numerical_columns"]:
             
             # skip derived columns; they need main columns (inc. nulls) generated first
-            if num_col in self.spec_dict["derived_columns"]:
+            if num_col in (self.spec_dict.get("derived_columns", dict()) or dict()):
                 continue
 
             anon_df[num_col] = generate_continuous_column(
@@ -285,7 +302,7 @@ class newExhibit:
                 self.spec_dict["columns"][uuid_col_name]["miss_probability"],
                 dist,
                 self.spec_dict["metadata"]["random_seed"]
-            )
+            ).astype("category")
 
             anon_df.insert(0, uuid_col_name, uuid_col)
 
@@ -406,8 +423,20 @@ class newExhibit:
             if number_dropped > 0:
                 print(f"WARNING: Deleted {number_dropped} duplicates.")
                 anon_df = anon_df.loc[~duplicated_idx, :].reset_index(drop=True)
+
+        #10) APPLY ML MODELS; EXPERIMENTAL; NOT TESTED BY CI TO AVOID ADDING ML
+        # DEPENDENCIES TO THE BUILD
+        if models := self.spec_dict.get("models", None): #pragma: no cover
+
+            for model_name, params in models.items():
+                # load the model object
+                with open(package_dir("models", f"{model_name}.pickle"), "rb") as f:
+                    model = dill.load(f)
+
+                # apply the model to the anon_df
+                anon_df = model.apply_model(anon_df, **params)
             
-        #10) SAVE THE GENERATED DATASET AS CLASS ATTRIBUTE FOR EXPORT
+        #11) SAVE THE GENERATED DATASET AS CLASS ATTRIBUTE FOR EXPORT
         self.anon_df = anon_df
 
     def write_data(self): # pragma: no cover
