@@ -11,6 +11,8 @@ import textwrap
 import pandas as pd
 import numpy as np
 
+from exhibit.core.constants import ORIGINAL_VALUES_REGEX
+
 # Exhibit imports
 from ..utils import get_attr_values, package_dir
 from ..sql import query_anon_database
@@ -67,7 +69,7 @@ class CategoricalDataGenerator:
 
                 u_linked_df = generate_user_linked_anon_df(
                     spec_dict=self.spec_dict,
-                    linked_group=linked_group,
+                    linked_cols=linked_group[1],
                     num_rows=self.num_rows
                 )
 
@@ -193,14 +195,10 @@ class CategoricalDataGenerator:
         #capture categorical-only information
         paired_cols = col_attrs["paired_columns"]
         anon_set = col_attrs["anonymising_set"]
-        root_anon_set = anon_set.split(".")[0]
+        orig_vals = col_attrs["original_values"]
 
-        #special case if anonymising set isn't defined - assume regex
-        if root_anon_set not in self.fixed_anon_sets:
-            print(textwrap.dedent(f"""
-            WARNING: Anonymising set for {col_name} not recognized.
-            Assuming regex pattern and uniform random distribution.
-            """))
+        #generate values based on a regular expression specified in the anonymising_set
+        if isinstance(orig_vals, str) and orig_vals == ORIGINAL_VALUES_REGEX:
             return generate_regex_column(anon_set, col_name, self.num_rows)  
 
         #values were stored in SQL; randomise based on uniform distribution
@@ -247,7 +245,8 @@ class CategoricalDataGenerator:
 
         #missing data is the last row
         repl = sql_df[col_name].unique()
-        aliased_df = orig_df.replace(orig_df[col_name].values[:-1], repl)
+        aliases = dict(zip(orig_df[col_name].values[:-1], repl))
+        aliased_df = orig_df.applymap(lambda x: aliases.get(x, x))
         self.spec_dict["columns"][col_name]["original_values"] = aliased_df
 
         #we ignore Missing Data probability when we originally create the variable
@@ -281,21 +280,36 @@ class CategoricalDataGenerator:
         else:
             table_name, *sql_column = anon_set.split(".")
             sql_df = query_anon_database(table_name, sql_column, uniques)
-            #rename sql_df columns to be same as original + paired; zip is 
-            #only going to pair up columns up to the shorter list!
-            sql_df.rename(
-                columns=dict(zip(
-                    sql_df.columns,
-                    [col_name] + paired_cols
-                )),
-                inplace=True
-            )
+
+        #if sql df is an anonymising set with different column names, like mountaints,
+        #we want to rename them to the actual column names used in the spec;
+        #alternatively, if the sql df is a lookup and column there match the spec, we
+        #make sure to take those columns that match.
+        if set([col_name] + paired_cols).issubset(set(sql_df.columns)):
+            sql_df = sql_df[[col_name] + paired_cols]
+
+        #rename sql_df columns to be same as original + paired; zip is 
+        #only going to pair up columns up to the shorter list!
+        sql_df.rename(
+            columns=dict(zip(
+                sql_df.columns,
+                [col_name] + paired_cols
+            )),
+            inplace=True
+        )
 
         #2) GENERATE ANONYMISED ROWS
         if complete:
-            anon_df = sql_df
+            anon_df = sql_df.drop(columns="probability_vector", errors="ignore")
         else:
-            idx = self.rng.choice(len(sql_df), self.num_rows)
+            if "probability_vector" in sql_df.columns:
+                probs = sql_df["probability_vector"].astype(float).values
+                probs = probs / probs.sum()
+                sql_df.drop(columns="probability_vector", inplace=True)
+                idx = self.rng.choice(a=len(sql_df), p=probs, size=self.num_rows)
+            else:
+                idx = self.rng.choice(len(sql_df), self.num_rows)
+
             anon_list = [sql_df.iloc[x, :].values for x in idx]
             anon_df = pd.DataFrame(columns=sql_df.columns, data=anon_list)
 
