@@ -31,6 +31,7 @@ class MissingDataGenerator:
 
         self.spec_dict = spec_dict
         self.data = data
+        self.dtypes = data.dtypes
         self.nan_data = data
         self.wt = spec_dict.get("weights_table", None)
 
@@ -83,7 +84,9 @@ class MissingDataGenerator:
             if linked_col_groups and linked_col_groups[0][0] == 0:
                 user_linked_cols = linked_col_groups[0][1]
                 if col_name in user_linked_cols:
-                    self.nan_data[col_name].replace(MISSING_DATA_STR, pd.NA, inplace=True)
+                    self.nan_data[col_name] = (
+                        self.nan_data[col_name].map(
+                            lambda x: pd.NA if x == MISSING_DATA_STR else x))
                     continue
             
             miss_pct = self.spec_dict["columns"][col_name]["miss_probability"]
@@ -155,7 +158,7 @@ class MissingDataGenerator:
         # only go through this step if there are nulls in categorical columns
         # and the spec_dict includes numerical columns that would be affected
         # otherwise, return early.
-        cat_cols = self.spec_dict["metadata"]["categorical_columns"]
+        cat_cols = list(self.spec_dict["metadata"]["categorical_columns"])
         num_cols = (
             set(self.spec_dict["metadata"]["numerical_columns"]) -
             set(self.spec_dict.get("derived_columns", {}).keys()))
@@ -214,9 +217,10 @@ class MissingDataGenerator:
             self.nan_data.loc[mask, num_col] = scaled_new_series.values
 
         # replace Missing data back with np.nan
-        self.nan_data.replace({MISSING_DATA_STR : np.nan}, inplace=True)
+        self.nan_data = self.nan_data.applymap(
+            lambda x: np.nan if x == MISSING_DATA_STR else x)
 
-        return self.nan_data
+        return self.nan_data.astype(self.dtypes)
 
     def _find_columns_with_linked_missing_data(self):
         '''
