@@ -76,7 +76,10 @@ def save_predefined_linked_cols_to_db(df, id):
 
     # convert the original, prefixed values first to positional labels
     # and then just to numerical IDs
-    temp_df = prefixed_df.replace(orig_label_to_pos_label).replace(pos_label_to_id)
+    temp_df = (prefixed_df
+            .applymap(lambda x: orig_label_to_pos_label.get(x, x))
+            .applymap(lambda x: pos_label_to_id.get(x, x)))
+
     label_matrix = np.unique(temp_df.values, axis=0).astype(np.intc)
 
     # make sure column names don't have spaces
@@ -123,7 +126,8 @@ def add_prefix(df, sep="__"):
         
     return pd.DataFrame(data_dict)
 
-def generate_user_linked_anon_df(spec_dict, linked_group, num_rows):
+def generate_user_linked_anon_df(
+    spec_dict, linked_cols, num_rows, starting_col_matrix=None):
     '''
     Main function to generated user-defined linked columns.
 
@@ -131,56 +135,63 @@ def generate_user_linked_anon_df(spec_dict, linked_group, num_rows):
     ----------
     spec_dict    : dictionary
         specification plus internal keys, like _rng
-    linked_group : tuple
+    linked_cols : list
         there can be only one user-linked group (0, [linked_col_1, linked_col_2, ])
     num_rows     : int
         number of rows to generate
+    starting_col_matrix : np.Array shaped (num_rows, len(linked_cols))
+        the matrix is either filled with None values or pre-populated if the function
+        is run multiple times (like when regenerating values after applying custom 
+        actions like make_same)
 
     Returns
     -------
     Data Frame with linked columns
     '''
     
-    linked_cols = linked_group[1]
     table_id = spec_dict["metadata"]["id"]
     rng = spec_dict["_rng"]
     lookup, matrix = get_lookup_and_matrix_from_db(table_id)
     new_label_lookup, proba_lookup = build_new_lookups(spec_dict, linked_cols, lookup)
+    # DANGER WHEN REVERSING THE DICT - SAME VALUES IN MULTIPLE COLUMNS WILL BE LOST
+    rev_label_lookup = {key:value for value, key in new_label_lookup.items()}
     # linked columns dispersion list
     lcd = [spec_dict["columns"][col]["dispersion"] for col in linked_cols]
 
-    # initialise the first column without regard to dispersion value; the initial column
-    # uses exact probabilities as defined in the spec / equalised if taken from DB
+    # if re-creating linked values from a pre-generated sequence, reverse the dict to
+    # get the numerical mapping as expected, also changing the dtype for performance.
 
-    init_probs = np.array([proba_lookup[x] for x in np.unique(matrix[:, 0])])
-    # make sure probs are normalized
-    if init_probs.sum() != 1:
-            init_probs /= init_probs.sum()
+    if starting_col_matrix is not None:
+       starting_col_matrix = (
+        pd.DataFrame(starting_col_matrix)
+        .fillna(MISSING_DATA_STR)
+        .applymap(lambda x: rev_label_lookup.get(x, x)).values.astype(np.int16))
 
-    init_col_vals = (
-            rng
-                .choice(a=np.unique(matrix[:, 0]), p=init_probs, size=num_rows)
-                .reshape(-1, 1)
-    )
+    else:
+        starting_col_matrix = np.full(
+            shape=(num_rows, len(linked_cols)), fill_value=-1)
 
     # multiprocessing only on unix
     if sys.platform != "win32":
         with Pool(processes=4) as pool:
 
             new_rows = pool.map(
-                partial(process_row, matrix, proba_lookup, lcd, rng), init_col_vals
+                partial(process_row, matrix, proba_lookup, lcd, rng),
+                starting_col_matrix
                 )
     else: #pragma: no cover
 
         new_rows = []
 
         for i in range(num_rows):
-            new_row = process_row(matrix, proba_lookup, lcd, rng, init_col_vals[i])
+            new_row = process_row(
+                matrix, proba_lookup, lcd, rng, starting_col_matrix[i])
             new_rows.append(new_row)
 
     new_matrix = np.stack(new_rows)
 
-    new_df = pd.DataFrame(new_matrix, columns=linked_cols).replace(new_label_lookup)
+    new_df = pd.DataFrame(
+        new_matrix, columns=linked_cols).applymap(lambda x: new_label_lookup.get(x, x))
 
     return new_df
 
@@ -195,7 +206,8 @@ def get_lookup_and_matrix_from_db(table_id):
 
     return lookup, matrix
 
-def process_row(label_matrix, proba_lookup, lcd, rng, accumulated_array, i=0):
+def process_row(
+    label_matrix, proba_lookup, lcd, rng, ref_array, acc_array=None, i=0):
     '''
     Recursive function to generate new rows of data from the 
     existing linked matrix. It's possible the function will be 
@@ -220,8 +232,10 @@ def process_row(label_matrix, proba_lookup, lcd, rng, accumulated_array, i=0):
         list with dispersion values for each column in linked_columns
     rng               : np.rng
         shared RNG generator
-    accumulated_array : np.array
-        array of encoded values that make up a single row of the linked data
+    ref_array         : np.Array
+        array of either None values or pre-populated with existing df values
+    acc_array         : np.Array
+        accummulated array that is being processed and returned
     i                 : integer
         a counter in case we need to reduce the sequence size to check for valid
         combinations to determine the next valid value 
@@ -230,11 +244,15 @@ def process_row(label_matrix, proba_lookup, lcd, rng, accumulated_array, i=0):
     -------
     np.array of a single row with encoded column values
     '''
+
+    if acc_array is None:
+        acc_array = np.array([])
       
-    arr_len = accumulated_array.shape[0]
+    arr_len = len(acc_array)
+    ref_arr_len = len(ref_array)
     
     if arr_len == label_matrix.shape[1]:
-        return accumulated_array
+        return acc_array
 
     # if there are no valid targets due to dispersion throwing in a non-valid target,
     # rather than continue checking the full array (which will always fail to produce
@@ -242,34 +260,46 @@ def process_row(label_matrix, proba_lookup, lcd, rng, accumulated_array, i=0):
     # to counter i and increase until you exhaust the prior possibilities. The fallback
     # is that there will always be valid targets for previous sequence length = 1 aka
     # from one column to the next.
-
-    mask = np.all(label_matrix[:, i:arr_len] == accumulated_array[i:], axis=1)
+    
+    _ref_array = np.where(ref_array == -1, label_matrix, ref_array)
+    mask = np.all(label_matrix[:, i:ref_arr_len] == _ref_array[:, i:], axis=1)
 
     valid_targets = np.unique(label_matrix[mask, arr_len])
-    all_targets = np.unique(label_matrix[:, arr_len])
 
     if len(valid_targets) == 0:
 
         i = i + 1
-        return process_row(label_matrix, proba_lookup, lcd, rng, accumulated_array, i)
+        return process_row(
+            label_matrix, proba_lookup, lcd, rng, ref_array, acc_array, i)
         
     target_proba = np.array([proba_lookup[x] for x in valid_targets])
     # make sure the probabilities sum up to 1
     target_proba = target_proba * (1 / sum(target_proba))
 
-    # optional dispersion; add noise by including non-valid targets (if exist)
-    non_valid_targets = np.setdiff1d(all_targets, valid_targets)
     # take dispersion from the spec
     dispersion = lcd[arr_len]
 
-    if len(non_valid_targets) > 0 and rng.random() < dispersion:
-        next_val = rng.choice(a=non_valid_targets, size=1)
-    else:
-        next_val = rng.choice(a=valid_targets, p=target_proba, size=1)
+    # default is to pick a random valid target
+    next_val = rng.choice(a=valid_targets, p=target_proba, size=1)[0]
+
+    # except when it's already pre-generated
+    if ref_array[arr_len] != -1:
+        next_val = ref_array[arr_len]
+
+    # or dispersion is in effect; this part is expensive so only calculate if needed
+    elif dispersion and rng.random() < dispersion:
+        all_targets = np.unique(label_matrix[:, arr_len])
+        non_valid_targets = np.setdiff1d(all_targets, valid_targets)
+        if len(non_valid_targets) > 0:
+            next_val = rng.choice(a=non_valid_targets, size=1)[0]
+
+    new_array = np.append(acc_array, next_val)
+
+    # update the ref_array to capture the just generated value
+    if ref_array[arr_len] == -1:
+        ref_array[arr_len] = next_val
     
-    new_array = np.append(accumulated_array, next_val)
-    
-    return process_row(label_matrix, proba_lookup, lcd, rng, new_array)
+    return process_row(label_matrix, proba_lookup, lcd, rng, ref_array, new_array)
 
 def build_new_lookups(spec_dict, linked_cols, original_lookup):
     '''
@@ -306,9 +336,12 @@ def build_new_lookups(spec_dict, linked_cols, original_lookup):
                 [MISSING_DATA_STR]
             )
             orig_vals = pd.DataFrame(data={col:orig_vals_sorted})
-            prob_vector = [1 / orig_vals.shape[0]] * orig_vals.shape[0]
+            prob_vector = np.ones(orig_vals.shape[0])
+            # when values are from DB, set the miss probability to reflect the spec
+            prob_vector[-1] = spec_dict["columns"][col]["miss_probability"]
+            prob_vector /= prob_vector.sum()
 
-        if not prob_vector:
+        if prob_vector is None:
             prob_vector = orig_vals["probability_vector"].values
         
         pos_labels_temp = [f"{col}__{x}" for x in range(len(orig_vals[col].values))]
